///////////////////////////////////////////////////////////////////////////////
// File: RTSPSession.cpp
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2014 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/Admin/Admin.h"

#include "RTSPSession.h"

#include "JetByteTools/CoreTools/FileUtils.h"
#include "JetByteTools/CoreTools/StringUtils.h"
#include "JetByteTools/CoreTools/DebugTrace.h"

#include "JetByteTools/SocketTools/IStreamSocket.h"

#pragma hdrstop

//#define RTSP_SESSION_DEBUG

#ifdef RTSP_SESSION_DEBUG
#define DEBUGGING_CONNECTIONS(_e) JetByteTools::Core::OutputEx(_e)
#else
#define DEBUGGING_CONNECTIONS(_e)
#endif

///////////////////////////////////////////////////////////////////////////////
// Using directives
///////////////////////////////////////////////////////////////////////////////

using JetByteTools::Core::DumpData;
using JetByteTools::Core::CStringConverter;

using JetByteTools::IO::CSmartBuffer;

using JetByteTools::Socket::IStreamSocket;
using JetByteTools::Socket::ShutdownSend;

using JetByteTools::Streaming::CRTSP_1_0_RequestParser;
using JetByteTools::Streaming::CRTSPServer;
using JetByteTools::Streaming::IRTSPServer;
using JetByteTools::Streaming::IManageRTSPResources;

using std::string;

///////////////////////////////////////////////////////////////////////////////
// CRTSPSession
///////////////////////////////////////////////////////////////////////////////

CRTSPSession::CRTSPSession(
   const IManageRTSPResources::SupportedTransports supportedTransports,
   IManageRTSPResources &resourceManager,
   IStreamSocket &socket,
   const bool terminateActiveSessionsOnConnectionClose)
   :  CRTSP_1_0_RequestParser(static_cast<IRTSPServer&>(*this)),
      CRTSPServer(supportedTransports, resourceManager, *this, socket.GetRemoteAddress(), terminateActiveSessionsOnConnectionClose),
      m_socket(socket)
{

}

// Implement IInterleavedRTPTCPStream

void CRTSPSession::AddRef()
{
   m_socket.AddRef();
}

void CRTSPSession::Release()
{
   m_socket.Release();
}

void CRTSPSession::Close()
{
   m_socket.Shutdown(ShutdownSend);
}

void CRTSPSession::Abort()
{
   m_socket.AbortConnection();
}

bool CRTSPSession::TryWrite(
   CSmartBuffer &packet)
{
   DEBUGGING_CONNECTIONS(_T("Outgoing RTSP Data:\r\n") + DumpData(packet->GetMemory(), packet->GetUsed(), 60));

   return m_socket.TryWrite(packet);
}

void CRTSPSession::Write(
   const BYTE *pData,
   const DataLength dataLength)
{
   DEBUGGING_CONNECTIONS(_T("Outgoing RTSP Data:\r\n") + CStringConverter::AtoT(reinterpret_cast<const char *>(pData), dataLength));

   m_socket.Write(pData, dataLength);
}

///////////////////////////////////////////////////////////////////////////////
// End of file: RTSPSession.cpp
///////////////////////////////////////////////////////////////////////////////
