#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: RTSPSession.h
/////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2014 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/StreamingTools/RTSP_1_0_RequestParser.h"
#include "JetByteTools/StreamingTools/RTSPServer.h"
#include "JetByteTools/StreamingTools/IInterleavedRTPTCPStream.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Socket
   {
      class IStreamSocket;
   }
}

///////////////////////////////////////////////////////////////////////////////
// CRTSPSession
///////////////////////////////////////////////////////////////////////////////

class CRTSPSession :
   public JetByteTools::Streaming::CRTSP_1_0_RequestParser,
   private JetByteTools::Streaming::CRTSPServer,
   private JetByteTools::Streaming::IInterleavedRTPTCPStream
{
   public:

      CRTSPSession(
         JetByteTools::Streaming::IManageRTSPResources::SupportedTransports supportedTransports,
         JetByteTools::Streaming::IManageRTSPResources &resourceManager,
         JetByteTools::Socket::IStreamSocket &socket,
         bool terminateActiveSessionsOnConnectionClose);

      CRTSPSession(
         const CRTSPSession &rhs) = delete;

      CRTSPSession &operator=(
         const CRTSPSession &rhs) = delete;

      using CRTSPServer::AbortActiveSessions;

      using CRTSPServer::PauseActiveSessions;

      using CRTSPServer::ResumeActiveSessions;

   private :

      // Implement IInterleavedRTPTCPStream

      void AddRef() override;

      void Release() override;

      void Close() override;

      void Abort() override;

      bool TryWrite(
         JetByteTools::IO::CSmartBuffer &packet) override;

      void Write(
         const BYTE *pData,
         DataLength dataLength) override;

      JetByteTools::Socket::IStreamSocket &m_socket;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: RTSPSession.h
///////////////////////////////////////////////////////////////////////////////
