#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: CommandLine.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/CoreTools/CommandLine.h"

#include "JetByteTools/IOTools/BufferAllocator.h"

#include "JetByteTools/SocketTools/ListenBacklog.h"
#include "JetByteTools/SocketTools/AddressIPv4.h"
#include "JetByteTools/SocketTools/StreamSocketConnectionManager.h"
#include "JetByteTools/SocketTools/FlowControlStreamSocketConnectionFilter.h"
#include "JetByteTools/SocketTools/ConsecutiveSocketOperations.h"
#include "JetByteTools/SocketTools/PerSocketBufferPoolingLimit.h"
#include "JetByteTools/SocketTools/ZeroByteReadConfiguration.h"

///////////////////////////////////////////////////////////////////////////////
// CCommandLine
///////////////////////////////////////////////////////////////////////////////

class CCommandLine
{
   public :

      typedef unsigned short Count;

      enum ServerType
      {
         TCPServer                                                   = 0x00000001,
         TCPClient                                                   = 0x00000002,
         UDPServer                                                   = 0x00000004,
         SupportsPerfCounters                                        = 0x00000010,
         SupportsSendRecvBufSize                                     = 0x00000020,
         UDPMulticastServer                                          = 0x00000044,
         SupportsZeroByteRead                                        = 0x00000100,
         SupportsIOPoolEx                                            = 0x00000400,
         SupportsPageAllocator                                       = 0x00001000,
         UDPConnectionServer                                         = 0x00020004,
         DoesNotSupportWelcomeMessage                                = 0x00100000,
         SupportsFlowControl                                         = 0x01000000,
         SupportsProcessingDelay                                     = 0x10000000
      };

      CCommandLine(
         const DWORD argc,
         const char **argv,
         const JetByteTools::Core::_tstring &serverName,
         DWORD type);

      CCommandLine(
         const CCommandLine &rhs) = delete;

      virtual ~CCommandLine() = default;

      CCommandLine &operator=(
         const CCommandLine &rhs) = delete;

      bool Parse();

      JetByteTools::Core::_tstring EventNamePrefix() const;

      JetByteTools::Core::_tstring Server() const;

      JetByteTools::Socket::CAddressIPv4::Port Port() const;

      bool DisplayDebug() const;

      Count NumberOfIOThreads() const;

      JetByteTools::Socket::ListenBacklog ListenBacklog() const;

      Count SocketPoolSize() const;

      JetByteTools::Socket::PerSocketBufferPoolingLimit MaxBuffersPooledPerSocket() const;

      JetByteTools::Socket::ConsecutiveSocketOperations MaxConsecutiveOperationsOnOneSocket() const;

      JetByteTools::IO::CBufferAllocator::BufferCount BufferPoolSize() const;

      JetByteTools::IO::IBuffer::BufferSize BufferSize() const;

      bool NoWelcomeMessage() const;

      Count MaxConnections() const;

      bool RegisterCounters() const;

      bool UnRegisterCounters() const;

      bool HasMulticastAddress() const;

      JetByteTools::Core::_tstring MulticastAddress() const;

      unsigned long RecvBufferSize() const;

      unsigned long SendBufferSize() const;

      JetByteTools::Socket::ZeroByteReadConfiguration GetZeroByteReadConfiguration() const;

      JetByteTools::IO::CBufferAllocator::AllocationFlags GetAllocationFlags() const;

      JetByteTools::Milliseconds ConnectionTimeout() const;

      JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::BufferCount MaxBuffers() const;

      JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::PendingWriteCount MaxPendingWrites() const;

      JetByteTools::Socket::CFlowControlStreamSocketConnectionFilter::Percentage NotificationPercentageStepSize() const;

      JetByteTools::Milliseconds ProcessingDelay() const;

      DWORD NumberOfEntriesPerCall() const;

      void ShowHelp(
         const JetByteTools::Core::_tstring &message);

   protected :

      const DWORD m_argc;

      const char **m_argv;

      JetByteTools::Core::CCommandLine m_parser;

      static Count StringToCount(
         const JetByteTools::Core::_tstring &stringValue);

      static unsigned long StringToUnsignedLong(
         const JetByteTools::Core::_tstring &stringValue);

   private :

      virtual void ShowMoreHelp();

      const JetByteTools::Core::_tstring m_serverName;

      const ServerType m_type;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: CommandLine.h
///////////////////////////////////////////////////////////////////////////////
